from argparse import ArgumentParser
from pathlib import Path
import pandas as pd
import typing as T
import yaml
import sys

REQUEST_PREFIXES = {
    "GSVREQUEST": "gsv",
    "OPAREQUEST": "opa",
}


def _get_parser() -> ArgumentParser:
    parser = ArgumentParser()
    parser.add_argument(
        "path",
        help="Path for mother request",
        type=str,
    )
    return parser


def _extract_from_df(df: pd.DataFrame, extract: T.Sequence[str]):
    return df[["Application", "Request No.", *extract]]


def parse_mother_request(path: str | Path, extract: T.Optional[T.Sequence[str]] = None):
    with open(path, "r", encoding="utf-8") as rf:
        mother_request = yaml.safe_load(rf)
    requests = []
    for app, app_requests in mother_request.items():
        for i, request_i in app_requests.items():
            request_dct = {
                "Application": app,
                "Request No.": i,
            }
            for request_type, request in request_i.items():
                request_prefix = REQUEST_PREFIXES[request_type]
                request_dct.update(
                    {f"{request_prefix}_{key}": value for key, value in request.items()}
                )
            requests.append(request_dct)
    df = pd.DataFrame(requests)
    if extract is not None:
        return _extract_from_df(df, extract)
    return df


if __name__ == "__main__":
    args = _get_parser().parse_args()
    path = Path(args.path)
    df = parse_mother_request(path)
